----------------------------------------------------------------------------------
-- Gerome Cacho, Jose Hurtado
-- Final Project CPE 133/01-02
-- Fall 2017
-- Prof: Hummel
-- Proj: Count to 15 Binary Game using Basys Board Switches with Live Timer
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Game is
    Port ( Clock : in STD_LOGIC;
           Switch : in STD_LOGIC_VECTOR (3 downto 0);
           Ein : in STD_LOGIC;
           Rst : in STD_LOGIC;
           LED : out STD_LOGIC_Vector (15 downto 0); 
           Anode : out STD_LOGIC_VECTOR (3 downto 0);
           Cathode : out STD_LOGIC_VECTOR (7 downto 0));     
end Game;

architecture Behavioral of Game is

signal PS : Std_logic_vector (4 downto 0) := "00000"; -- signal for our FSM Present State
signal NS : Std_logic_vector (4 downto 0) := "00000"; -- signal for our FSM Next State
signal Play : std_logic := '0'; -- signal for our enable for the stopwatch logic
signal digit1 : integer := 0; -- signal for Digit 1 (Right Most)
signal digit2 : integer := 0; -- signal for Digit 2 (Second to Right)
signal digit3 : integer := 0; -- signal for Digit 3 (Second to Left)
signal digit4 : integer := 0; -- signal for Digit 4 (Left Most)
signal Fail : std_logic := '0'; -- signal for our Fail condition, once toggled high, the fail sequence will initiate

begin

    State_Decider: process (Clock, Rst, Fail) -- Controls the States
    begin
        If Rst = '1' then -- Reset takes precedence, over anything else
            PS <= "00000";
        elsif Fail = '1' then -- Fail only goes live if the timer reaches 100 seconds
            PS <= "11011";
        elsif (Rising_edge(Clock)) then -- the Basys board has a 100MHz Clock, which on every rising edge of the square wave pulse, we assign our stages
                PS <= NS;
        end if;       
    end process;
        
    Switch_Logic: process (PS, Switch, Ein) -- Logic controlling the switches, which is dependant on the stages. LED will light up, telling you what number to count to next
    begin 
        NS <= PS; --fixes a PS and NS latch
        Case (PS) is
            When "00000" => --Pre-set Stage
                LED <= "0000000000000000";
                Play <= '0'; -- Play must be defined in every case here, or else creates a latch
                if Switch = "0000" and Ein = '1' then
                    NS <= "00001";
                end if;
            when "00001" => -- Set Stage
                LED <= "0000000000000001";
                Play <= '0';
                if Switch = "0001" then
                    NS <= "00010";
                end if;
            when "00010" => --Stage where someone is playing, switch is at 0001
                LED <= "0000000000000011";
                Play <= '1';
                if Switch = "0010" then
                    NS <= "00011";
                end if;
            when "00011" =>
                LED <= "0000000000000111";
                Play <= '1';
                if Switch = "0011" then
                    NS <= "00100";
                end if;
            when "00100" =>
                LED <= "0000000000001111";
                Play <= '1';
                if Switch = "0100" then
                    NS <= "00101";
                end if;
            when "00101" =>
                LED <= "0000000000011111";
                Play <= '1';
                if Switch = "0101" then
                    NS <= "00110";
                end if;
            when "00110" =>
                LED <= "0000000000111111";
                Play <= '1';
                if Switch = "0110" then
                    NS <= "00111";
                end if;
            when "00111" =>
                LED <= "0000000001111111";
                Play <= '1';
                if Switch = "0111" then
                    NS <= "01000";
                end if;
            when "01000" =>
                LED <= "0000000011111111";
                Play <= '1';
                if Switch = "1000" then
                    NS <= "01001";
                end if;
            when "01001" =>
                LED <= "0000000111111111";
                Play <= '1';
                if Switch = "1001" then
                    NS <= "01010";
                end if;
            when "01010" =>
                LED <= "0000001111111111";
                Play <= '1';
                if Switch = "1010" then
                    NS <= "01011";
                end if;
            when "01011" =>
                LED <= "0000011111111111";
                Play <= '1';
                if Switch = "1011" then
                    NS <= "01100";
                end if;
            when "01100" =>
                LED <= "0000111111111111";
                Play <= '1';
                if Switch = "1100" then
                    NS <= "01101";
                end if;
            when "01101" =>
                LED <= "0001111111111111";
                Play <= '1';
                if Switch = "1101" then
                    NS <= "01110";
                end if;
            when "01110" =>
                LED <= "0011111111111111";
                Play <= '1';
                if Switch = "1110" then
                    NS <= "01111";
                end if;
            when "01111" =>
                LED <= "0111111111111111";
                Play <= '1';
                if Switch = "1111" then
                    NS <= "10000";
                end if;
            when "10000" => -- Win Stage, time recorded on the 7 segment
                LED <= "1111111111111111";
                Play <= '0';
            when "11011" => -- Failed Stage, only occurs when timer reaches 100 seconds
                LED <= "1010101010101010";
                Play <= '0';
            when others => -- Failsafe
                LED <= "0000000000000000";
                Play <= '0';
        end case;
    end process;
    
    Stopwatch : process (Play, Rst, Clock)
        variable Count: integer := 0;
    begin
        if Rst = '1' then
            Count := 0;
            Digit1 <= 0; -- Controls the first digit of the seven segment
            Digit2 <= 0; -- Controls the second digit of the seven segment
            Digit3 <= 0; -- Controls the third digit of the seven segment
            Digit4 <= 0; -- Controls the fourth digit of the seven segment
            Fail <= '0'; -- Fail is our fail signal, which once triggered, will go to the stage "11011"
        elsif Play = '0' then -- If Play is off, then count will be the held to the value as it was
            Count := Count;
        elsif Play = '1' and rst = '0' then
            if (rising_edge(Clock)) then
                Count := Count + 1;
                if Count = 1000000 then -- Count will only increment Digit0 every hundreth of a second
                    Digit1 <= Digit1 + 1;
                    Count := 0;
                    if Digit1 = 9 then
                        Digit2 <= Digit2 + 1;
                        Digit1 <= 0;
                        if Digit2 = 9 then
                            Digit3 <= Digit3 + 1;
                            Digit2 <= 0;
                            if Digit3 = 9 then
                                Digit4 <= Digit4 + 1;
                                Digit3 <= 0;
                                if Digit4 = 9 then -- Once the last digit rolls over
                                    digit4 <= 0;
                                    Fail <= '1';
                                end if;
                            end if;
                        end if;
                    end if;
                end if;
            end if;
        end if;
    end process;
    
    Seven_Seg_Display : process (Clock, Digit1, Digit2, Digit3, Digit4) -- Controls the 7 Segment Display on the Basys Board
            variable counting : integer := 0;
        begin
            if (rising_edge(Clock)) then
                Counting := counting +1;
                If Counting = 1 then
                    Anode <= "1110"; --Utilize Right-Most Digit of 7-Segment Only
                    If Digit1 = 0 then --0
                        Cathode <= "00000011";
                    elsif Digit1 = 1 then --1
                        Cathode <= "10011111";
                    elsif Digit1 = 2 then --2
                        Cathode <= "00100101";
                    elsif Digit1 = 3 then --3
                        Cathode <= "00001101";
                    elsif Digit1 = 4 then --4
                        Cathode <= "10011001";
                    elsif Digit1 = 5 then --5
                        Cathode <= "01001001";
                    elsif Digit1 = 6 then --6
                        Cathode <= "01000001";
                    elsif Digit1 = 7 then --7
                        Cathode <= "00011111";
                    elsif Digit1 = 8 then --8
                        Cathode <= "00000001";
                    elsif Digit1 = 9 then--9
                        Cathode <= "00011001";
                    end if;
                elsif Counting = 400000 then -- Refreshes the Second Right-Most Segment
                    Anode <= "1101";
                    If Digit2 = 0 then --0
                        Cathode <= "00000011";
                    elsif Digit2 = 1 then --1
                        Cathode <= "10011111";
                    elsif Digit2 = 2 then --2
                        Cathode <= "00100101";
                    elsif Digit2 = 3 then --3
                        Cathode <= "00001101";
                    elsif Digit2 = 4 then --4
                        Cathode <= "10011001";
                    elsif Digit2 = 5 then --5
                        Cathode <= "01001001";
                    elsif Digit2 = 6 then --6
                        Cathode <= "01000001";
                    elsif Digit2 = 7 then --7
                        Cathode <= "00011111";
                    elsif Digit2 = 8 then --8
                        Cathode <= "00000001";
                    elsif Digit2 = 9 then--9
                        Cathode <= "00011001";
                    end if;
                elsif Counting = 800000 then -- Refreshes the Second Left-Most Segment
                    Anode <= "1011";
                    If Digit3 = 0 then --0
                        Cathode <= "00000010";
                    elsif Digit3 = 1 then --1
                        Cathode <= "10011110";
                    elsif Digit3 = 2 then --2
                        Cathode <= "00100100";
                    elsif Digit3 = 3 then --3
                        Cathode <= "00001100";
                    elsif Digit3 = 4 then --4
                        Cathode <= "10011000";
                    elsif Digit3 = 5 then --5
                        Cathode <= "01001000";
                    elsif Digit3 = 6 then --6
                        Cathode <= "01000000";
                    elsif Digit3 = 7 then --7
                        Cathode <= "00011110";
                    elsif Digit3 = 8 then --8
                        Cathode <= "00000000";
                    elsif Digit3 = 9 then--9
                        Cathode <= "00011000";
                    end if;
                elsif Counting = 1200000 then --Refreshes the Left-Most digit of the 7 Segment
                    Anode <= "0111";
                    If Digit4 = 0 then --0
                        Cathode <= "00000011";
                    elsif Digit4 = 1 then --1
                        Cathode <= "10011111";
                    elsif Digit4 = 2 then --2
                        Cathode <= "00100101";
                    elsif Digit4 = 3 then --3
                        Cathode <= "00001101";
                    elsif Digit4 = 4 then --4
                        Cathode <= "10011001";
                    elsif Digit4 = 5 then --5
                        Cathode <= "01001001";
                    elsif Digit4 = 6 then --6
                        Cathode <= "01000001";
                    elsif Digit4 = 7 then --7
                        Cathode <= "00011111";
                    elsif Digit4 = 8 then --8
                        Cathode <= "00000001";
                    elsif Digit4 = 9 then--9
                        Cathode <= "00011001";
                    end if;
                elsif Counting = 1600000 then -- Resets Counting to 0, which repeats this process
                    Counting := 0;
                end if;
            end if;
        end process;
end Behavioral;